#See http://rubyforge.org/tracker/?func=detail&aid=19509&group_id=5650&atid=21859
require 'test/unit'
require 'ftools'
#
#Extend the class TestCase with additional methods/assertions
#
class Test::Unit::TestCase
  #
  #Takes the content of the file 'filename' and compares it with 'actual' like in assert_equal.
  #If 'filname' doesn't exist, the failure 
  # Reference file <#{filename}> missing
  #is returned.
  #
  #'folder_for_failure' will contain all results with differences.
  #
  #Example of the usage:
  #    assert_equal_filecontent( "expected/test_section.html", text.to_doc(:html))
  #
  #What will happen:
  #1) text.to_doc(:html) is the test. It creates some HTML-Text
  #2) "expected/test_section.html" is read and compared to text.to_doc(:html)
  #2a)  If the file is missing -> error
  #2b) If it is the same -> fine 
  #2c) If there are differences:
  #3) A message is given (like in assert_equal)
  #4) A folder "failure#{Date.today}" is created if not already exist
  #5) The file 'test_section.html' with the result is created in "failure#{Date.today}/"
  #6) I can use a compare tool to compare the expected result and the real result.
  #
  #Recommendation to build up your test.
  #1) Create two folders: 'expected' and 'failure'
  #2) Define your assertion with a non-existing filename
  #3) Run the test with folder_for_failure = 'failure'
  #4) You will get a failure (filename is missing).
  #5) Copy the file in 'failure' to 'expected'
  #6) Rerun again, you hav no failure (hopefully ;-) )
  def assert_equal_filecontent( filename, actual, folder_for_failure = "failure#{Date.today}", message = nil )
    #
    #Make the tests
    if File.exist?(filename)
      expected = File.read(filename) 
      full_message = build_message(message, "<?> expected (#{filename}) but was\n<?>.\n", expected, actual).to_s
    else
      full_message = "Reference file <#{filename}> missing"
    end

    #Write the real result to a file if a failure folder is given.
    if folder_for_failure and expected  != actual
      File.makedirs(folder_for_failure) if ! File.directory?(folder_for_failure)
      File.open( "#{folder_for_failure}/#{File.basename(filename)}", 'w'){|f|
        f << actual
      }
      full_message << "\n\t-> Build <#{folder_for_failure}/#{File.basename(filename)}>"
    end

    if File.exist?(filename)
      assert_block( full_message ){ expected  == actual }
    else
      assert_block( full_message ){ false }
    end
  end
  #
end

if $0 == __FILE__
#Some content to be tested.
DUMMYTEXT = <<dummytext
Some text to be tested.
More text.
dummytext

#
#~ require 'test/testunit/test_assertions'
#This is a test in the test -> we need test_assertions.rb, not only test/unit
#test_assertions.rb is not in path of tests -> full path, please adapt for your installation.
#
require "C:/Program Files/ruby/doc/ruby/ruby-1.8.6/test/testunit/test_assertions.rb"
class MyTest < Test::Unit::TC_Assertions
  def test_ok()
    #Build reference data
    File.makedirs('expected')
    File.open('expected/test.txt', 'w'){|f| f << DUMMYTEXT }
    #Make test
    assert_equal_filecontent( 'expected/test.txt', DUMMYTEXT )
  end    
  def test_no_reference_file()
    check_fails(%r{Reference file <expected/not_available.txt> missing}){
      assert_equal_filecontent( 'expected/not_available.txt', DUMMYTEXT, nil )
    }
  end
  def test_build_reference_file()
    #Initial expected file
    File.makedirs('expected')
    File.open('expected/test_build.txt', 'w'){|f| f << DUMMYTEXT*2 }
    #
    folder_for_failure  = 'failure'
    #~ assert_equal(false, File.exist?("#{folder_for_failure}/test.txt" ))
    
    #First check. There is a difference, in failure we get the correct result
    check_fails(%r{.*expected \(expected/test_build.txt\) but was.*-> Build <failure/test_build.txt}m){
      assert_equal_filecontent( 'expected/test_build.txt', DUMMYTEXT, folder_for_failure )
    }
    #Check for file with correkt result
    assert_equal(true, File.exist?("#{folder_for_failure}/test_build.txt" ))
    #Check with new file (normally you would copy the file to 'expected'
    assert_equal_filecontent( "#{folder_for_failure}/test_build.txt", DUMMYTEXT, nil )
  end
end #class MyTest
end #$0 == __FILE__